#!/usr/bin/wish
# bnchat implemented in tcl, begun 5/26/99.
#
# Renamed "battlechat" on 6/4/99, Robert D. Nelson <ronelson@vt.edu>

global reply
global channel
global conn
global username
global title
global target
global squelchlist
global server

set title "BattleChat"

set EID_SHOWUSER               1001
set EID_JOIN                   1002
set EID_LEAVE                  1003
set EID_WHISPER                1004
set EID_TALK                   1005
set EID_BROADCAST              1006
set EID_CHANNEL                1007
set EID_USERFLAGS              1009
set EID_WHISPERSENT            1010
set EID_CHANNELFULL            1013
set EID_CHANNELDOESNOTEXIST    1014
set EID_CHANNELRESTRICTED      1015
set EID_INFO                   1018
set EID_ERROR                  1019
set EID_EMOTE                  1023
set EID_NULL                   2000
set EID_NAME                   2010

set reply ""
set channel ""
set conn ""
set username ""
set target ""
set squelchlist ""


# Load external sources.
source config.tcl


proc quit { } {

  global conn

  if ![ file exists $conn ] {
    exit
  } else {
    flush $conn; close $conn
    exit
  }
  
}

proc init_servers { m } {

  global server
  global serverlist
  
  set server ""

# Test, and if the server list doesn't exist, make it.
  if { ![file exists "~/.battlechat/servers.lst"] } {
    if { ![file exists "~/.battlechat"] } {
      file mkdir "~/.battlechat"
    }
    set fileId [open "~/.battlechat/servers.lst" w]
    puts $fileId "exodus.battle.net"
    close $fileId
  }

# Create and fill the file since it doesn't exist.
#    puts $fileId "*servers.list: 216 209 207 206 exodus"
#    puts $fileId "*servers.216: 1"
#    puts $fileId "*216.1: 216.32.73.174"
#    puts $fileId "*servers.209: 1 2 3"
#    puts $fileId "*209.1: 209.67.136.174"
#    puts $fileId "*209.2: 209.67.136.172"
#    puts $fileId "*209.3: 209.67.136.170"
#    puts $fileId "*servers.207: 1 2 3"
#    puts $fileId "*207.1: 207.69.194.210"
#    puts $fileId "*207.2: 207.69.194.189"
#    puts $fileId "*207.3: 207.138.34.3"
#    puts $fileId "*servers.206: 1"
#    puts $fileId "*206.1: 206.79.254.192"
#    puts $fileId "*servers.exodus: 1"
#    puts $fileId "*exodus.1: exodus.battle.net"
#    close $fileId

  set fileId [open "~/.battlechat/servers.lst" r]
  gets $fileId server
  close $fileId

  $m add cascade -label "216 servers..." -menu $m.216
  set n [menu $m.216 -tearoff 0]
  $n add command -label "216.32.73.174" -command { serverset "216.32.73.174" }

  $m add cascade -label "209 servers..." -menu $m.209
  set n [menu $m.209 -tearoff 0]
  $n add command -label "209.67.136.174" -command { serverset "209.67.136.174" }
  $n add command -label "209.67.136.172" -command { serverset "209.67.136.172" }
  $n add command -label "209.67.136.170" -command { serverset "209.67.136.170" }

  $m add cascade -label "207 servers..." -menu $m.207
  set n [menu $m.207 -tearoff 0]
  $n add command -label "207.69.194.210" -command { serverset "207.69.194.210" }
  $n add command -label "207.69.194.189" -command { serverset "209.69.194.189" }
  $n add command -label "207.138.34.3" -command { serverset "207.138.34.3" }

  $m add cascade -label "exodus servers..." -menu $m.exodus
  set n [menu $m.exodus -tearoff 0]
  $n add command -label "exodus.battle.net" -command { serverset "exodus.battle.net" }

}

proc serverset { name } {

  global server
  
  set server $name
  
  if { ![file exists "~/.battlechat/servers.lst"] } {
    if { ![file exists "~/.battlechat"] } {
      file mkdir "~/.battlechat"
    }
  }
  set fileId [ open "~/.battlechat/servers.lst" w ]
  puts $fileId $server
  close $fileId

}

proc init_menu { } {

  global back
  global server

  frame .menu -relief raised -bd 2 -bg $back
  pack .menu -side top -padx 1 -pady 1 -fill x
  menubutton .menu.f -text File -underline 0 -menu .menu.f.menu

  set m [menu .menu.f.menu -tearoff 0]
  $m add command -label "Save Buffer" -underline 0 -command { menufSaveBuffer }
  $m add separator
  $m add command -label Exit -underline 1 -command { quit }

  menubutton .menu.c -text Connection -underline 0 -menu .menu.c.menu

  set m [menu .menu.c.menu -tearoff 0]
  $m add command -label Disconnect -underline 0 -command { menucDisconnect }
  $m add separator
  $m add command -label "Edit Server List" -underline 0 -command { menucEditServerList }

  menubutton .menu.s -text Settings -underline 0 -menu .menu.s.menu

  set m [menu .menu.s.menu -tearoff 0]
  $m add cascade -label "Servers..." -underline 0 -menu $m.servers
  set n [menu $m.servers -tearoff 0]
  init_servers $n

  pack .menu.f .menu.c .menu.s -side left -fill y

}

proc init_top { } {

  frame .topwindow -relief flat -bd 2
  text .topwindow.chat -width 72 -height 15 -relief raised
  .topwindow.chat config -yscrollcommand ".topwindow.chatsb set"
  bind .topwindow.chat <Button-1> { focus .topwindow.chat }

  scrollbar .topwindow.chatsb -command ".topwindow.chat yview"
  text .topwindow.who -relief raised -width 18
  init_colors
  
  .topwindow.who config -yscrollcommand ".topwindow.whosb set"
  scrollbar .topwindow.whosb -command ".topwindow.who yview"

  pack .topwindow.chat .topwindow.chatsb .topwindow.who .topwindow.whosb -side left -fill y
  pack .topwindow -side top -fill y

  bindtags .topwindow.chat { Text .topwindow.chat . all }
  bindtags .topwindow.who { Text .topwindow.chat . all }
  bind .topwindow.chat <Button-1> { focus .frame.name }
  bind .topwindow.who <Button-1> { focus .frame.name }

}

proc init_login { } {

  global back

  frame .frame -relief raised -bd 2 -bg $back

  label .frame.label -text "Name: " -relief flat -padx 8
  entry .frame.name -width 20 -textvariable username -bg $back
  bind .frame.name <Return> { focus .frame.pass }
  label .frame.label2 -text "Password: " -relief flat -padx 8
  entry .frame.pass -width 20 -textvariable userpass -bg $back
  bind .frame.pass <Return> { pass_ok }
  button .frame.submit -text "OK" -command { pass_ok } -padx 8 -bg $back

  bindtags .frame.submit { Button all .frame.submit . }
  bindtags .frame.name { Entry all .frame.name . }
  bind .frame.submit <Return> { pass_ok }
  bind .frame.submit <Tab> { focus .frame.name }
  bind .frame.submit <Shift-Tab> { focus .frame.pass }
  bind .frame.name <Shift-Tab> { focus .frame.submit }

  pack .frame.label .frame.name .frame.label2 .frame.pass .frame.submit -side left -fill y
  pack .frame -fill x

  focus .frame.name

}

proc init_online { } {

  global back
  global textcomm
  global conn
  
  destroy .frame
  frame .frame -relief flat -bd 2 -bg $back
  .frame config -cursor top_left_arrow

  entry .frame.entry -width 63 -textvariable textcomm -bg $back
  button .frame.squelch -text "Squelch" -command { squelch } -padx 8 -bg $back
  button .frame.whisper -text "Whisper" -command { whisper } -padx 8 -bg $back

  bind .frame.entry <Return> { updatechat $textcomm ; set textcomm "" }
  bind . <Alt-w> { whisper }
  bind . <Alt-s> {squelch }
  bind . <Next> { .topwindow.chat yview scroll 1 pages }
  bind . <Prior> { .topwindow.chat yview scroll -1 pages }

  pack .frame.entry .frame.squelch .frame.whisper -side left -fill both -ipadx 8
  pack .frame -fill x
  fileevent $conn readable do_loop

  bindtags .frame.whisper { Button all .frame.whisper . }
  bindtags .frame.entry { Entry all .frame.entry . }
  bind .frame.whisper <Tab> { focus .frame.entry }
  bind .frame.whisper <Shift-Tab> { focus .frame.squelch }
  bind .frame.entry <Shift-Tab> { focus .frame.whisper }

  bind .frame.whisper <space> { focus .frame.entry }
  bind .frame.squelch <space> { focus .frame.entry }

  bind .topwindow.chat <Button-1> { focus .frame.entry }
  bind .topwindow.who <Button-1> { focus .frame.entry }

  focus .frame.entry

}

proc pass_ok {} {

#  global back
  global username
  global userpass
  global conn
  global textcomm
  global server

  set conn [socket $server 6112]
  puts $conn "c"; flush $conn        

  set reply [gets $conn]
  if { [string match refused [lindex [split $reply] end]] } {
    .topwindow.chat insert end "Connection refused.\n" {red}
    return
  }
  gets $conn; gets $conn; gets $conn; gets $conn; gets $conn
  puts $conn "$username"; flush $conn
  set reply [gets $conn]
  puts $conn "$userpass"; flush $conn
  gets $conn; set reply [gets $conn]

  init_online

}

proc do_loop { } {

  global color
  global count
  global reply
  global conn
  
  if [eof $conn] {
    catch {close $conn}
    .topwindow.chat insert end "\n\nConnection Broken!\n\n"
    return
  } else {
    set reply [gets $conn]
    parse_message
  }

}

proc addToWho { speaker game } {

  set char $speaker
  set list [.topwindow.who get 1.0 end]
  set del [lsearch -exact $list $char]
  if { $del == -1} {
    switch -exact -- $game {
      OP { .topwindow.who insert 1.0 "$speaker\n" {OP}  }
      DRTL { .topwindow.who insert end "$speaker\n" {DRTL}  }
      STAR { .topwindow.who insert end "$speaker\n" {STAR}  }
      SEXP { .topwindow.who insert end "$speaker\n" {SEXP}  }
      CHAT { .topwindow.who insert end "$speaker\n" {CHAT}  }
      W2BN { .topwindow.who insert end "$speaker\n" {W2BN}  }
      default { .topwindow.who insert end "$speaker code:$game\n" }
    }
  }

}

proc delFromWho { speaker } {

  global target
  
  set char $speaker
  set list [.topwindow.who get 1.0 end]
  set del [expr [lsearch -exact $list $char] +1]
  .topwindow.who delete $del.0 [expr $del +1].0
  if { $target == $char } { set target "" }

}

proc clearWho { } {

  .topwindow.who delete 1.0 end

}

proc addOp { speaker } {

  .topwindow.who insert 1.0 "$speaker\n" {OP}
  
}

proc squelch {} {

  global back
  global target
  global squelchlist
    
  set n1 [lindex [split [.topwindow.who dump -mark insert]] 2]
  set n2 [lindex [split $n1 .] 0]
  set target [.topwindow.who get $n2.0 $n2.end]
  if { $target == "" } {
    .topwindow.chat insert end "You need to choose someone to squelch first.\n" {error}
  } else {
    if { $squelchlist == "" } {
      lappend squelchlist $target
      updatechat "/squelch $target"
    } else {
      if { [set item [lsearch -exact $squelchlist $target]] != -1 } {
        lreplace squelchlist $item $item
        updatechat "/unsquelch $target"
      } else {
        lappend squelchlist $target
        updatechat "/squelch $target"
      }
    }
  }

}

proc whisper { } {
 
  global back
  global textcomm
  global target
  
  set n1 [lindex [split [.topwindow.who dump -mark insert]] 2]
  set n2 [lindex [split $n1 .] 0]
  set target [.topwindow.who get $n2.0 $n2.end]
  if { $target == "" } {
    .topwindow.chat insert end "You need to choose someone to whisper to first.\n" {error}
  } else {
    updatechat "/w $target $textcomm"
  }
  .frame.entry delete 0 end

}

proc updatechat { textcomm } {

  global conn
  
  if { $textcomm == "/quit" } {
    catch [close $conn]; exit
  }
  if { $textcomm != "" } {
    puts $conn $textcomm; flush $conn
    .topwindow.chat yview moveto 1
  }
  
}

proc parse_message { } {

  global title
  global color
  global count
  global reply
  global channel
  global username
  global conn

  set str $reply  
  set id [lindex [split $str] 0]
  set str [string range $str 5 end]
  
  switch -exact -- $id {
    1000 { }
    1001 {
      set speaker [lindex [split $str] 1]; #EID_SHOWUSER
      if { [string index [lindex [split $str] 2] 3] == "2" } {
        set game OP
      } else {
        set game [lindex [split $str] 3]
        set game [string trimleft $game \[]
        set game [string trimright $game \]]
      }
      addToWho $speaker $game
    }
    1002 {
      set speaker [lindex [split $str] 1]; #EID_JOIN
      set game [lindex [split $str] 3]
      set game [string trimleft $game \[]
      set game [string trimright $game \]]
      addToWho $speaker $game
      .topwindow.chat insert end "\n$speaker has joined the channel." {join}
      .topwindow.chat yview scroll 1 units
    }
    1003 {
      set speaker [lindex [split $str] 1]; #EID_LEAVE
      delFromWho $speaker
      .topwindow.chat insert end "\n$speaker has left the channel." {leave}
      .topwindow.chat yview scroll 1 units
    }
    1004 {
      set speaker [lindex [split $str] 1]; #EID_WHISPER
      set count [expr [string length $speaker] + 14]
      set str [string range $str $count end]
      set str [string trim $str \"]
      .topwindow.chat insert end "\n<From: $speaker> $str" {whisper}
      .topwindow.chat yview moveto 1
    }
    1005 {
      set speaker [lindex [split $str] 1]; #EID_TALK
      set count [expr [string length $speaker] + 11]
      set str [string range $str $count end]
      set str [string trim $str \"]
      .topwindow.chat insert end "\n<$speaker> " {talk}
      .topwindow.chat insert end "$str"
      .topwindow.chat yview moveto 1
    }
    1006 {
      set str [string range $str 5 end]; #EID_BROADCAST
      set color "red"
      .topwindow.chat insert end "\nBROADCAST code: $str" {red}
      .topwindow.chat yview scroll 1 units
      puts $reply
    }
    1007 {
      set str [string range $str 8 end]; #EID_CHANNEL
      set str [string trim $str \"]
      set channel $str
      wm title . "$title - $channel"
      .topwindow.chat insert end "\nJoining channel $channel..." {channel}
      .topwindow.chat yview scroll 1 units
      clearWho
    }
    1009 {
      set speaker [lindex [split $str] 1]; #EID_USERFLAGS
      if { [string index [lindex [split $str] 2] 3] == "2" } {
        delFromWho $speaker
        addOp $speaker
      }
    }
    1010 {
      set speaker [lindex [split $str] 1]; #EID_WHISPERSENT
      set count [expr [string length $speaker] + 14]
      set str [string range $str $count end]
      set str [string trim $str \"]
      .topwindow.chat insert end "\n<To: $speaker> $str" {whispersent}
      .topwindow.chat yview moveto 1
    }
    1013 {
      set str [string range $str 5 end]; #EID_CHANNELFULL
      .topwindow.chat insert end "\nCHANNELFULL code: $str" {red}
      .topwindow.chat yview moveto 1
      puts $reply
    }
    1014 {
      set str [string range $str 5 end]; #EID_CHANNELDOESNOTEXIST
      .topwindow.chat insert end "\nCHANNELDOESNOTEXIST code: $str" {red}
      .topwindow.chat yview moveto 1
      puts $reply
    }
    1015 {
      set str [string range $str 5 end]; #EID_CHANNELRESTRICTED
      .topwindow.chat insert end "\nCHANNELRESTRICTED code: $str" {red}
      .topwindow.chat yview moveto 1
      puts $reply
    }
    1018 {
      set str [string range $str 5 end]; #EID_INFO
      set str [string trim $str \"]
      .topwindow.chat insert end "\n$str" {info}
      .topwindow.chat yview moveto 1
    }
    1019 {
      set str [string range $str 6 end]; #EID_ERROR
      set str [string trim $str \"]
      .topwindow.chat insert end "\n$str" {error}
      .topwindow.chat yview moveto 1
    }
    1023 {
      set speaker [lindex [split $str] 1]; #EID_EMOTE
      set count [expr [string length $speaker] + 12]
      set str [string range $str $count end]
      set str [string trim $str \"]
      .topwindow.chat insert end "\n$speaker $str" {emote}
      .topwindow.chat yview moveto 1
    }
    2000 {
      set reply ""; #EID_NULL
    }
    2010 {
      set reply ""; #EID_NAME
    }
    default {
      set color "blue"
      if { [string first / $reply] == 0 } {
        set reply ""
      } else {
        .topwindow.chat insert end "\n<$username> " {talked}
        .topwindow.chat insert end "$reply"
        .topwindow.chat yview moveto 1
      }
    }
  }    

}

proc menufSaveBuffer { } {

  set file [ tk_getSaveFile -initialdir "~/" -initialfile "battlechat.buffer" -title "Save As..." ]
  
  if [string match $file ""] {
    # They hit cancel or hit OK with a null name field somehow
    return;
  }

  if [ file exists $file ] {
    file delete -force $file
  }
  
  if [ catch { open $file w 0600 } fileId ] {
    puts stderr "Cannot open $file: $fileId"
  } else {
    puts $fileId [ .topwindow.chat get 0.0 end ]
    close $fileId
  }

}

proc menucDisconnect { } {

  global conn

  catch {close $conn}
  .topwindow.chat delete 1.0 end
  .topwindow.who delete 1.0 end
  destroy .frame
  init_login

}

proc menucEditServerList { } {

  puts "proc menucEditServerList"

}


proc init { } {

  global title
  global server
  
  bind . <Alt-x> quit
  wm title . $title

  init_menu
  init_top
  init_login

  if { $server == "" } { 
    set server exodus.battle.net
  }

}

global back
set back lightgrey

init
